#include "stm32f10x.h"
#define USART_STDIO
#include "usart/usart.h"
#include <stdio.h>
#include "counter/counter.h"
#include "spi/spi.h"
#include "i2c/i2c.h"
#include "ds1307/ds1307.h"
#include "utils/delay.h"
#include "tft9341/tft9341.h"
#include "tft9341/tft9341_cache.h"
#include "tft9341/tft9341_cache_vbuf.h"
#include "graphics/graphics.h"
#include "graphics/painter.h"
#include "graphics/font_5x8_utf8.h"
#include "graphics/font_10x16_utf8.h"
#include "font_droid_sans_mono_8x16_utf8.h"
#include "fixed/fixed32.h"
#include "gui/gui.h"
#include "gui/gui_object.h"
#include "gui/gui_widget.h"
#include "gui/gui_label.h"
#include "gui/gui_number_label.h"
#include "gui/gui_checkbox.h"
#include "gui/gui_anim_bitmap.h"
#include "clock_gui.h"
#include <string.h>
#include <stdlib.h>

#define USART_WRITE_BUFFER_SIZE 128
static uint8_t usart_write_buffer[USART_WRITE_BUFFER_SIZE];

#define USART_READ_BUFFER_SIZE 128
static uint8_t usart_read_buffer[USART_READ_BUFFER_SIZE];

static usart_buf_t usart_buf;

static counter_t counter = 0;

static counter_t time_counter = 0;
static counter_t anim_counter = 0;

static i2c_bus_t i2c;

static ds1307_t rtc;

static spi_bus_t spi;

static tft9341_t tft;
#define TFT_PIXEL_SIZE 2
#define TFT_WIDTH 320
#define TFT_HEIGHT 240

#define TFT_CACHE_BUFS_COUNT 2
#define TFT_CACHE_BUF0_PIXELS 60
#define TFT_CACHE_BUF1_PIXELS 100
#define TFT_CACHE_BUF0_SIZE (TFT_CACHE_BUF0_PIXELS * TFT_PIXEL_SIZE)
#define TFT_CACHE_BUF1_SIZE (TFT_CACHE_BUF1_PIXELS * TFT_PIXEL_SIZE)

static uint8_t tft_cache_buf_data0[TFT_CACHE_BUF0_SIZE];
static uint8_t tft_cache_buf_data1[TFT_CACHE_BUF1_SIZE];

static tft9341_cache_buffer_t tft_cache_bufs[TFT_CACHE_BUFS_COUNT] = {
    make_tft9341_cache_buffer(tft_cache_buf_data0, TFT_CACHE_BUF0_SIZE),
    make_tft9341_cache_buffer(tft_cache_buf_data1, TFT_CACHE_BUF1_SIZE)
};
static tft9341_cache_t tft_cache = make_tft9341_cache(&tft, TFT_PIXEL_SIZE, tft_cache_bufs, TFT_CACHE_BUFS_COUNT, TFT9341_ROW_COL_REVERSE_MODE);
static graphics_vbuf_t graph_vbuf = make_tft9341_cache_vbuf();
static graphics_t graphics = make_tft9341_cache_graphics(&tft_cache, &graph_vbuf, TFT_WIDTH, TFT_HEIGHT, GRAPHICS_FORMAT_RGB_565);
//static painter_t painter = make_painter(&graphics);

static const font_bitmap_t font_5x8_utf8_bitmaps[] = {
    make_font_bitmap(32, 127, font_5x8_utf8_part0_data, FONT_5X8_UTF8_PART0_WIDTH, FONT_5X8_UTF8_PART0_HEIGHT, GRAPHICS_FORMAT_BW_1_V),
    make_font_bitmap(0xb0, 0xb0, font_5x8_utf8_part1_data, FONT_5X8_UTF8_PART1_WIDTH, FONT_5X8_UTF8_PART1_HEIGHT, GRAPHICS_FORMAT_BW_1_V),
    make_font_bitmap(0x400, 0x451, font_5x8_utf8_part2_data, FONT_5X8_UTF8_PART2_WIDTH, FONT_5X8_UTF8_PART2_HEIGHT, GRAPHICS_FORMAT_BW_1_V)
};
static font_t font5x8 = make_font(font_5x8_utf8_bitmaps, 3, 5, 8, 1, 0);

const font_bitmap_t font_10x16_utf8_bitmaps[] = {
    make_font_bitmap(32, 127, font_10x16_utf8_part0_data, FONT_10X16_UTF8_PART0_WIDTH, FONT_10X16_UTF8_PART0_HEIGHT, GRAPHICS_FORMAT_BW_1_V),
    make_font_bitmap(0xb0, 0xb0, font_10x16_utf8_part1_data, FONT_10X16_UTF8_PART1_WIDTH, FONT_10X16_UTF8_PART1_HEIGHT, GRAPHICS_FORMAT_BW_1_V),
    make_font_bitmap(0x400, 0x451, font_10x16_utf8_part2_data, FONT_10X16_UTF8_PART2_WIDTH, FONT_10X16_UTF8_PART2_HEIGHT, GRAPHICS_FORMAT_BW_1_V)
};
static font_t font10x16 = make_font(font_10x16_utf8_bitmaps, 3, 10, 16, 1, 0);

const font_bitmap_t font_8x16_utf8_bitmaps[] = {
    make_font_bitmap(32, 127, font_droid_sans_mono_8x16_utf8_32_127_data, FONT_DROID_SANS_MONO_8X16_UTF8_32_127_WIDTH, FONT_DROID_SANS_MONO_8X16_UTF8_32_127_HEIGHT, GRAPHICS_FORMAT_BW_1_V),
    make_font_bitmap(0xb0, 0xb0, font_droid_sans_mono_8x16_utf8_0xb0_0xb0_data, FONT_DROID_SANS_MONO_8X16_UTF8_0XB0_0XB0_WIDTH, FONT_DROID_SANS_MONO_8X16_UTF8_0XB0_0XB0_HEIGHT, GRAPHICS_FORMAT_BW_1_V),
    make_font_bitmap(0x400, 0x451, font_droid_sans_mono_8x16_utf8_0x400_0x451_data, FONT_DROID_SANS_MONO_8X16_UTF8_0X400_0X451_WIDTH, FONT_DROID_SANS_MONO_8X16_UTF8_0X400_0X451_HEIGHT, GRAPHICS_FORMAT_BW_1_V)
};
static font_t font8x16 = make_font(font_8x16_utf8_bitmaps, 3, 8, 16, 0, 0);

// GUI.
#define MAKE_RGB565(r, g, b) TFT9341_MAKE_RGB565(r, g, b)

#define THEME_COLOR_BACK MAKE_RGB565(0xff, 0xff, 0xff)
#define THEME_COLOR_FRONT MAKE_RGB565(0x00, 0x00, 0x00)
#define THEME_COLOR_PANEL MAKE_RGB565(0xf8, 0xf8, 0xf8)//0xf0
#define THEME_COLOR_WIDGET MAKE_RGB565(0xe1, 0xe1, 0xe1)
#define THEME_COLOR_BORDER MAKE_RGB565(0xb1, 0xb1, 0xb1)
#define THEME_COLOR_FONT MAKE_RGB565(0x00, 0x00, 0x00)
#define THEME_COLOR_FOCUS MAKE_RGB565(0x00, 0x78, 0xd7)
#define THEME_COLOR_PRESSED MAKE_RGB565(0xcd, 0xe3, 0xf8)

#define CLOCK_BACK_COLOR MAKE_RGB565(0x00, 0x00, 0x00)
#define CLOCK_MONTHDAY_COLOR MAKE_RGB565(0xff, 0xff, 0x00)
#define CLOCK_MONTH_COLOR MAKE_RGB565(0xff, 0xff, 0x00)
#define CLOCK_WEEKDAY_COLOR MAKE_RGB565(0x00, 0xff, 0xff)
#define CLOCK_TIME_COLOR MAKE_RGB565(0x00, 0xff, 0x00)
#define CLOCK_TEMP_OUTD_COLOR MAKE_RGB565(0x00, 0x00, 0xff)
#define CLOCK_TEMP_IND_COLOR MAKE_RGB565(0xff, 0x00, 0x00)

static gui_theme_t theme = MAKE_GUI_THEME(
                    THEME_COLOR_BACK, THEME_COLOR_FRONT,
                    THEME_COLOR_PANEL, THEME_COLOR_WIDGET,
                    THEME_COLOR_BORDER, THEME_COLOR_FONT,
                    THEME_COLOR_FOCUS, THEME_COLOR_PRESSED,
                    &font5x8, &font10x16);
static gui_t gui = MAKE_GUI(&graphics, &theme);


/*
 * Обработчики прерываний.
 */

void USART1_IRQHandler(void)
{
    usart_buf_irq_handler(&usart_buf);
}

void SysTick_Handler(void)
{
    system_counter_tick();
}

void SPI1_IRQHandler(void)
{
    spi_bus_irq_handler(&spi);
}

void DMA1_Channel2_IRQHandler(void)
{
    spi_bus_dma_rx_channel_irq_handler(&spi);
}

void DMA1_Channel3_IRQHandler(void)
{
    spi_bus_dma_tx_channel_irq_handler(&spi);
}

bool spi_callback(void)
{
    return tft9341_spi_callback(&tft);
}

void I2C1_EV_IRQHandler(void)
{
    i2c_bus_event_irq_handler(&i2c);
}

void I2C1_ER_IRQHandler(void)
{
    i2c_bus_error_irq_handler(&i2c);
}

void DMA1_Channel6_IRQHandler(void)
{
    i2c_bus_dma_tx_channel_irq_handler(&i2c);
}

void DMA1_Channel7_IRQHandler(void)
{
    i2c_bus_dma_rx_channel_irq_handler(&i2c);
}

static bool i2c_callback(void)
{
    return true;
}

static void init_sys_counter(void)
{
    system_counter_init(1000);
    counter = system_counter_ticks();
    SysTick_Config(SystemCoreClock / 1000);
}

static void init_periph_clock(void)
{
    // AFIO.
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
    // USART.
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
    // DMA.
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
    // SPI.
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    // I2C.
    //RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
}

static void init_usart(void)
{
    GPIO_InitTypeDef gpio_tx =
        {.GPIO_Pin = GPIO_Pin_9, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_AF_PP};
    GPIO_InitTypeDef gpio_rx =
        {.GPIO_Pin = GPIO_Pin_10, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_IN_FLOATING};
    GPIO_Init(GPIOA, &gpio_tx);
    GPIO_Init(GPIOA, &gpio_rx);
    
    USART_InitTypeDef usart_is =
        {.USART_BaudRate = 115200, .USART_WordLength = USART_WordLength_8b, .USART_StopBits = USART_StopBits_1,
         .USART_Parity = USART_Parity_No, .USART_Mode = USART_Mode_Rx | USART_Mode_Tx, .USART_HardwareFlowControl = USART_HardwareFlowControl_None};
    USART_Init(USART1, &usart_is);
    USART_Cmd(USART1, ENABLE);
    
    usart_buf_init_t usartb_is = {.usart = USART1,
         .write_buffer = usart_write_buffer, .write_buffer_size = USART_WRITE_BUFFER_SIZE,
         .read_buffer = usart_read_buffer, .read_buffer_size = USART_READ_BUFFER_SIZE};
    usart_buf_init(&usart_buf, &usartb_is);
    usart_setup_stdio(&usart_buf);
    
    NVIC_SetPriority(USART1_IRQn, 2);
    NVIC_EnableIRQ(USART1_IRQn);
}

static void init_i2c(void)
{
    GPIO_InitTypeDef gpio_i2c =
        {.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7, .GPIO_Speed = GPIO_Speed_10MHz, .GPIO_Mode = GPIO_Mode_AF_OD};
    GPIO_Init(GPIOB, &gpio_i2c);
    
    //i2c_device_reset(I2C1);
    
    I2C_InitTypeDef i2c_is;
    i2c_is.I2C_Ack = I2C_Ack_Disable;
    i2c_is.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    i2c_is.I2C_ClockSpeed = 100000;
    i2c_is.I2C_DutyCycle = I2C_DutyCycle_2;
    i2c_is.I2C_Mode = I2C_Mode_I2C;
    i2c_is.I2C_OwnAddress1 = 0x55;
    
    I2C_Init(I2C1, &i2c_is);
    I2C_StretchClockCmd(I2C1, ENABLE);
    I2C_Cmd(I2C1, ENABLE);
    
    i2c_bus_init_t i2c_bus_is =
        { .i2c_device = I2C1, .dma_rx_channel = DMA1_Channel7, .dma_tx_channel = DMA1_Channel6 };
    
    i2c_bus_init(&i2c, &i2c_bus_is);
    i2c_bus_set_callback(&i2c, i2c_callback);
    
    NVIC_SetPriority(I2C1_EV_IRQn, 1);
    NVIC_SetPriority(I2C1_ER_IRQn, 1);
    NVIC_EnableIRQ(I2C1_EV_IRQn);
    NVIC_EnableIRQ(I2C1_ER_IRQn);
    
    NVIC_SetPriority(DMA1_Channel6_IRQn, 1);
    NVIC_SetPriority(DMA1_Channel7_IRQn, 1);
    NVIC_EnableIRQ(DMA1_Channel6_IRQn);
    NVIC_EnableIRQ(DMA1_Channel7_IRQn);
}

static void init_spi(void)
{
    GPIO_InitTypeDef gpio_sck_mosi =
        {.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_7, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_AF_PP};
    GPIO_InitTypeDef gpio_miso =
        {.GPIO_Pin = GPIO_Pin_6, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_IN_FLOATING};
    
    GPIO_Init(GPIOA, &gpio_sck_mosi);
    GPIO_Init(GPIOA, &gpio_miso);
    
    SPI_InitTypeDef spi_is;
    SPI_StructInit(&spi_is);
    
    spi_is.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;
    spi_is.SPI_CPHA = SPI_CPHA_1Edge;
    spi_is.SPI_CPOL = SPI_CPOL_Low;
    spi_is.SPI_CRCPolynomial = 0;
    spi_is.SPI_DataSize = SPI_DataSize_8b;
    spi_is.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    spi_is.SPI_FirstBit = SPI_FirstBit_MSB;
    spi_is.SPI_Mode = SPI_Mode_Master;
    spi_is.SPI_NSS = SPI_NSS_Soft;
    
    SPI_Init(SPI1, &spi_is);
    SPI_Cmd(SPI1, ENABLE);
    
    spi_bus_init_t spi_bus_is;
    spi_bus_is.spi_device = SPI1;
    spi_bus_is.dma_rx_channel = DMA1_Channel2;
    spi_bus_is.dma_tx_channel = DMA1_Channel3;
    spi_bus_init(&spi, &spi_bus_is);
    
    spi_bus_set_callback(&spi, spi_callback);
    
    NVIC_SetPriority(SPI1_IRQn, 3);
    NVIC_EnableIRQ(SPI1_IRQn);
    
    NVIC_SetPriority(DMA1_Channel2_IRQn, 4);
    NVIC_SetPriority(DMA1_Channel3_IRQn, 4);
    NVIC_EnableIRQ(DMA1_Channel2_IRQn);
    NVIC_EnableIRQ(DMA1_Channel3_IRQn);
}

static void init_tft(void)
{
    GPIO_InitTypeDef gpio_dc = 
    {.GPIO_Pin = GPIO_Pin_12, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_Out_PP};
    GPIO_InitTypeDef gpio_ce = 
    {.GPIO_Pin = GPIO_Pin_11, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_Out_PP};
    GPIO_InitTypeDef gpio_rst = 
    {.GPIO_Pin = GPIO_Pin_10, .GPIO_Speed = GPIO_Speed_50MHz, .GPIO_Mode = GPIO_Mode_Out_PP};
    
    GPIO_Init(GPIOB, &gpio_dc);
    GPIO_Init(GPIOB, &gpio_ce);
    GPIO_Init(GPIOB, &gpio_rst);
    
    tft9341_init_t tft_init;
    
    tft_init.ce_gpio = GPIOB; tft_init.ce_pin = GPIO_Pin_11;
    tft_init.dc_gpio = GPIOB; tft_init.dc_pin = GPIO_Pin_12;
    tft_init.reset_gpio = GPIOB; tft_init.reset_pin = GPIO_Pin_10;
    tft_init.spi = &spi;
    tft_init.transfer_id = TFT9341_DEFAULT_TRANSFER_ID;
    
    tft9341_init(&tft, &tft_init);
    
    tft9341_reset(&tft);
    
    tft9341_madctl_t madctl;
    madctl.row_address_order = TFT9341_ROW_TOP_TO_BOTTOM;//TFT9341_ROW_BOTTOM_TO_TOP;
    madctl.col_address_order = TFT9341_COL_LEFT_TO_RIGHT;//TFT9341_COL_RIGHT_TO_LEFT;
    madctl.row_col_exchange = TFT9341_ROW_COL_REVERSE_MODE;
    madctl.vertical_refresh = TFT9341_REFRESH_TOP_TO_BOTTOM;
    madctl.color_order = TFT9341_COLOR_ORDER_BGR;
    madctl.horizontal_refresh = TFT9341_REFRESH_LEFT_TO_RIGHT;
    
    tft9341_set_madctl(&tft, &madctl);
    tft9341_set_pixel_format(&tft, TFT9341_PIXEL_16BIT, TFT9341_PIXEL_16BIT);
    tft9341_sleep_out(&tft);
    tft9341_display_on(&tft);
}

static void gui_set_clock_time(void)
{
    clock_gui_set_time(time_counter / 100, time_counter % 100);
}

static void init_gui(void)
{
    clock_gui_init(&gui);
    clock_gui_set_back_color(CLOCK_BACK_COLOR);
    clock_gui_set_monthday_color(CLOCK_MONTHDAY_COLOR);
    clock_gui_set_month_color(CLOCK_MONTHDAY_COLOR);
    clock_gui_set_weekday_color(CLOCK_WEEKDAY_COLOR);
    clock_gui_set_time_color(CLOCK_TIME_COLOR);
    clock_gui_set_temp_ind_color(CLOCK_TEMP_IND_COLOR);
    clock_gui_set_temp_outd_color(CLOCK_TEMP_OUTD_COLOR);
    //gui_repaint(&gui, NULL);
    gui_set_clock_time();
    clock_gui_set_monthday(1);
    clock_gui_set_month(0);
    clock_gui_set_weekday(0);
    clock_gui_set_temp_outd(fixed16_make_from_fract(-57, 10));
    clock_gui_set_temp_ind(fixed16_make_from_fract(254, 10));
}

static void gui_iter(void)
{
    clock_gui_animation_step();
}

int main(void)
{
    NVIC_SetPriorityGrouping(0x3);
    
    init_periph_clock();
    //init_led();
    init_sys_counter();
    init_usart();
    
    printf("STM32 MCU\r\n");
    
    init_i2c();
    init_spi();
    init_tft();
    
    init_gui();
    
    //time_counter = 0;
    anim_counter = system_counter_ticks();
    
    for(;;){
        if(system_counter_diff(&anim_counter) >= system_counter_ticks_per_sec() / 35){
            anim_counter = system_counter_ticks();
            gui_iter();
        }
        if(system_counter_diff(&counter) >= system_counter_ticks_per_sec()){
            counter = system_counter_ticks();
            
            clock_gui_toggle_time_sep();
            if(++ time_counter > 9999) time_counter = 0;
            gui_set_clock_time();
            
            printf("%d\r\n", ((int)counter));
        }
    }
    return 0;
}
